@@ -565,8 +565,6 @@ ipv4_getstate(struct interface *ifp)
 			return NULL;
 		}
 		TAILQ_INIT(&state->addrs);
-		state->buffer_size = state->buffer_len = state->buffer_pos = 0;
-		state->buffer = NULL;
 	}
 	return state;
 }
@@ -963,6 +961,5 @@ ipv4_free(struct interface *ifp)
 		TAILQ_REMOVE(&state->addrs, ia, next);
 		free(ia);
 	}
-	free(state->buffer);
 	free(state);
 }
